% user_functions: funciones definidas por el usuario
% Copyright (c) 2007-2008 Luis Pineda (http://leibniz.iimas.unam.mx/~luis)
% Copyright (c) 2009-     Luis Pineda (http://leibniz.iimas.unam.mx/~luis), 
%                         Elia Patricia Pavon  ()
%                         Ivan Meza   (http://turing.iimas.unam.mx/~ivanvladimir)


%Todas las funciones tienen como argumentos (EXCLUSIVAMENTE):
%            Dialogue_ID
%            Historia
%            Argumentos locales
%
%Todas las funciones deben asignar el valor mediante:
%
%	    assert(function_value(FUNC_VALUE))
%

:- use_module(library(random)).
:- use_module(library(system)).


solve_situation(Diag_ID, In_History, [[Sp_Act], Sp_Act]).
	
%Obtiene al acto retorico que sigue despues de contar cuantas preguntas se han hecho
get_rht_act(Diag_ID, In_History, [Rht_normal, Rht_last, Rht_no]) :-
	get_current_DM_history(Diag_ID, In_History, Current_DM_History),
	get_performed_sp_acts(Current_DM_History,Performed_Sp_Acts),
	length_list(Performed_Sp_Acts, Size),
    goto_act(Size, Rht_normal, Rht_last, Rht_no, Rht_Act),
	assert(function_value(Rht_Act)).

%Obtiene la siguiente situacion una vez contadas cuantas preguntas se han hecho
next_sit(Diag_ID, In_History, [Sit_l, Sit_s]) :-
	get_current_DM_history(Diag_ID, In_History, Current_DM_History),
	get_performed_sp_acts(Current_DM_History, Performed_Sp_Acts),
	length_list(Performed_Sp_Acts, Size),
    goto_sit(Size, Sit_l, Sit_s, Situation),
	assert(function_value(Situation)).

%Checa que la edad sea mayor a 6 para escoger la siguiente situacion o el siguiente acto retorico
age_check(Diag_ID, In_History, [Edad, Sigue, Para]) :-
    goto_next(Edad, Sigue, Para, Next),

	%assign function value
	Next_AS =.. [Next,Edad],
	assert(function_value(Next)).

%Evaluacion de las caracteristicas correctas o incorrectas de un determinado modelo de dialogo
eval_pregunta(Diag_ID, In_History, [Model, X, Rht_Act]) :-
	ja_caracteristicas(Model, C),
	get_options(C,X,Option),
	Next_Rht_Act =.. [Rht_Act, X, Option,Model],
	assert(function_value(Next_Rht_Act)).

%Nos dice si la carta escogida es la correcta segn el modelo de dialogo
get_rht_carta(Diag_ID, In_History, [Rht_ok, Rht_no, Carta, Model]) :-
	get_current_DM_history(Diag_ID, In_History, Current_DM_History),
	get_carta(Current_DM_History,Carta),
	equal(Rht_ok,Rht_no,Carta,Model,Rht,Valor),	
	print('Rht: '),print(Rht),nl,
	Next_Rht_Act =.. [Rht, Carta, Valor],
	assert(function_value(Next_Rht_Act)).

%Obtiene un modelo de dialogo al azar
get_random_topic(Diag_ID, In_History, [Pred, Topics, Sit]) :-
    Pred,
    get_random_element(Topics,Topic),
	NSit =.. [Sit,Topic],
	assert(function_value(NSit)).

%get rht act that takes into account past visited locations
next_rht_act(Diag_ID, In_History, [Local_Options, Rht_Act]) :-
	get_current_DM_history(Diag_ID, In_History, Current_DM_History),
	get_performed_sp_acts(Current_DM_History, Performed_Sp_Acts),
	difference(Local_Options, Performed_Sp_Acts, Options),
	Next_Rht_Act =.. [Rht_Act, Options],
	assert(function_value(Next_Rht_Act)).


% Funciones auxiliares
get_options(LC,E,Option) :-
		(member(E,LC)) ->
			Option = ok
		| otherwise ->
			Option = no.


get_carta([],X).
get_carta([ID:(_ => carta(X) :_,_,_:_=>_)|T],X).
get_carta([_|T],X) :-
	get_carta(T, X).

get_performed_sp_acts([],[]).
get_performed_sp_acts([ID:(_ => SP:_,TS,_)|T], [SP|In_Sp_Acts]) :-
	(TS == ts_1 ; TS == ts_2; TS == ts_3; TS == rs_1; TS == rs_2; TS == rs_3), 
	get_performed_sp_acts(T, In_Sp_Acts).
 
get_performed_sp_acts([_|T], Sp_Acts) :-
	get_performed_sp_acts(T, Sp_Acts).


%get motor speech act from current to requested locations
%The motor rhetorical act must be the out_rht of the only out_pair of an initial situation of a DM
move_from_prev_loc(Current_Location, History) :-
	reverse(History, Rev_History),
	get_previous_location(Rev_History, Previous_Location),
	assert(function_value(ma_1(Previous_Location,Current_Location))).

move_from_prev_loc(Current_Location, History) :- print('error: unknown current area.').

get_previous_location([], main).
get_previous_location([main:_|T], Previous_Location) :-
		get_previous_location(T, Previous_Location).
get_previous_location([Previous_Location:_|_], Previous_Location).

goto_act(X,A,B,C,A) :- X < 2.
goto_act(X,A,B,C,B) :- X = 2.
goto_act(X,A,B,C,C) :- X = 3.

goto_sit(X,D,E,D) :- X < 4.
goto_sit(X,D,E,E) :- X = 4.

goto_next(X,D,E,E) :- X =< 6.
goto_next(X,D,E,D) :- X > 6.

equal(A,B,X,Y,B,no) :- X \== Y.
equal(A,B,X,Y,A,ok) :- X == Y.
	
% Random functions 
get_random_element(L,E) :-
    length_list(L,Size),
    datime(datime(_,_,_,_,_,Seed)),
	getrand(rand(X,Y,_)),	
	setrand(rand(Seed,X,Y)),				
    random(0,Size,Id),
    idx_object(Id,L,E).


