% bindings.pl: Does bindigs 
% Copyright (c) 2004-  Luis Pineda (http://leibniz.iimas.unam.mx/~luis)


%%%%%%%%%%%%%%%%%%%%%%%%%%%% Utilites for variable bindings %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bind_vars([],TARGS).
bind_vars([HVAR|TVARS],[HARG|TARGS]) :-
	    HVAR = HARG,
	    bind_vars(TVARS,TARGS).

get_new_variables([],[]).
get_new_variables([H|T],[H|NT]) :-
	    atom(H),
	    get_new_variables(T, NT),!.
get_new_variables([H|T],[H|NT]) :-
	    number(H),
	    get_new_variables(T, NT),!.
get_new_variables([H|T],[NH|NT]) :-
	    var(H),
	    get_new_variables(T, NT),!.
get_new_variables([[H1|H2]|T],[[H1|NH2]|NT]) :-
        atom(H1),
	    get_new_variables(H2,NH2),
	    get_new_variables(T, NT),!.
get_new_variables([[H1|H2]|T],[[H1|NH2]|NT]) :-
        number(H1),
	    get_new_variables(H2,NH2),
	    get_new_variables(T, NT),!.
get_new_variables([[H1|H2]|T],[[NH1|NH2]|NT]) :-
        var(H1),
	    get_new_variables(H2,NH2),
	    get_new_variables(T, NT),!.




replace_variables([],[],E,E).
replace_variables([H|T],[NH|NT], Rht_Acts,NRht_Acts) :-
	    change_variable(H,NH,Rht_Acts,Next_Rht_Acts),
	    replace_variables(T,NT,Next_Rht_Acts,NRht_Acts).

change_variable(VAR,NVAR,[],[]).
change_variable(VAR,NVAR,[H|T],NEXPS) :-
	    change_var_exp(VAR,NVAR,H,NEXP),
	    change_variable(VAR,NVAR,T,REST_EXPS),
	    append([NEXP],REST_EXPS,NEXPS).

change_var_exp(VAR, NVAR, EXP, NEXP) :-
	    var(EXP),
	    VAR == EXP,
	    NEXP = NVAR.

change_var_exp(VAR, NVAR, EXP, EXP) :-
	    var(EXP).

change_var_exp(VAR, NVAR, EXP, NEXP) :-
	    nonvar(EXP),
	    EXP =.. [FUNC|ARGS],
	    change_var_args(VAR, NVAR, ARGS, NARGS),
	    NEXP =.. [FUNC|NARGS].

change_var_args(VAR, NVAR, [], []).
change_var_args(VAR, NVAR, [H|T], NARGS) :-
	   change_var_exp(VAR, NVAR, H, NEXP),
	   change_var_args(VAR, NVAR, T, REXPS),
	   append([NEXP],REXPS,NARGS).

%%%%%%% predicados de estructura de terminso %%%%%%%%%%%%%%%

ground_term(TERM,false) :- var(TERM).
ground_term(TERM,true) :- atom(TERM).
ground_term(EXP,VAL) :- 
		     nonvar(EXP),
		     EXP =.. [FUNC|ARGS],
		     ground_args(ARGS,VAL).
ground_term(EXP,false).

ground_args([],true).
ground_args([H|T],true) :-
		     ground_term(H,true),
		     ground_args(T,true).
ground_args(_,false).
