% diag_manager: Dialogue manager based on Functional Recursive Transtion Networks
%               It conects to the OAA architecture
% Copyright (c) 2007-2008 Luis Pineda (http://leibniz.iimas.unam.mx/~luis)
% Copyright (c) 2009-     Luis Pineda (http://leibniz.iimas.unam.mx/~luis), 
%                         Ivan Meza (http://turing.iimas.unam.mx/~ivanvladimir)
%
:- use_module(library(terms),all).
:- use_module(library(system)).
:- use_module(library(clpfd)).

%  ---- Executable setting ---------------------------------------
user:runtime_entry(start) :-
     va.


%  ---- OAA setting ----------------------------------------------
% Specific to sictus
:- use_module(spcompat).
:- assert(user:library_directory('$OAA2_HOME/src/oaalib/prolog')).

:- dynamic name/1.

% Declaration of capabilities
initial_solvables([]).

% The entry point for agent
runtime_entry(run) :- run.

% Conects to the OAA facilitator
start :-
    use_module(_,library('com_tcp'), all),
    use_module(_,library('oaa'), all),

    % Connect to facilitator:
    com_Connect(parent, [], _CInfo, CInfo),
    format('Connected to ~p.~n',[CInfo]),
    % Register default callback for incoming events (messages):
    oaa_RegisterCallback(app_do_event, user:oaa_AppDoEvent),
    % Register with facilitator:
    initial_solvables(S),
    oaa_Register(parent, manejador_dialogo, S, []).
    %app_init.

% Run OAA and dialogue manager.
app_init :-
	write('The agent did not respond.'), nl,
	current_output(Output), flush_output(Output),
	halt(0).

%  ---- Dialogue manager starts  --------------------------------
% Operators for dialogue models
:- op(600,xfx,':').
:- op(800,xfx,'=>').
:- op(900,xfx,'==>').


% Loading local libraries
:- consult(read_input).
:- consult(utilities).
:- consult(sets).
:- consult(bindings).
:- consult(loggin).
:- consult(cl_opts).


% Description for command line options
desc_cl([
    [
     ['','--language_mode',true, 'Language model to use (language_model.pl)'],
     ['','--rht_acts',true,      'Speech acts to use    (rht_act.pl)'],
     ['','--vs_acts',true,       'Vision acts to use    (vs_acts.pl)'],
     ['','--templates',true,     'Templates to use      (templates.pl)'],
     ['','--motor_actions',true, 'Motor actions to use  (motor_actions.pl)'],
     ['','--user_functions',true,'User functions        (user_functions.pl)'],
     [],
     ['-m','--mode',true,'Mode of executing\n\t\trun - OAA\n\t\ttest - Not OAA'],
     [],
     ['-l','--log',true,'Log file to save information (dm.log)'],
     ['-v','--verbose',true,'Verbose mode, [debug,info] (info)'],
     [],
     ['-c','--copyright',false,'Prints copyright'],
     ['','--license',false,'Prints license'],
     ['','--version',false,'Prints version'],
     ['-h','--help',false,'Prints this help']
    ],
    [['project_dir',true,'Directory with the project to load'],
     ['main_dialogue',true,'Main dialogue to execute'],
     ['dialogue_list_file',true,'File with list of dialogue to load']
    ],
    ['sicsuts -l diag_manager --goal va. -a [opts] project_dir main_dialogue dialogue_list_file']
]).

copyright('\n
-----------------------------------------------------------------------------
Copyright (c) 2007-2008 Luis Pineda (http://leibniz.iimas.unam.mx/~luis)
Copyright (c) 2009-     Luis Pineda (http://leibniz.iimas.unam.mx/~luis), 
                        Ivan Meza (http://turing.iimas.unam.mx/~ivanvladimir)
-----------------------------------------------------------------------------
').

license('\n
-----------------------------------------------------------------------------
Exclusivo para el uso del proyecto Golem, DCC, IIMAS, UNAM, Golem-Universum 
y el curso Inteligencia Artifical 2009 de la maestría en Ciencia e Ingeniría
de la Computación, UNAM. Queda prohibida la copia de este software
sin la autorización de los autores. Para información contactar a autores.
-------------------------------------------------------------------------
').
dm_version('0.0.2').

% Main predicate for the DM 
va :-  
        % Parsing command line options
        desc_cl(Desc_CL),
	    (prolog_flag(argv,CL)-> 
            (getOptsArgs(CL,Desc_CL,Opts,Args) ->
                (existsOptArg('-c','--copyright',Opts)-> 
                    copyright(CR),
                    print(CR),
                    halt
                |existsOptArg('','--license',Opts) ->
                    license(LS),
                    print(LS),
                    halt
                |existsOptArg('','--version',Opts) ->
                    dm_version(LS),
                    print('--------------------------------------------------------------------------'),nl,
                    print('Dialogue Manager based on Functional Recursive Transition Networks v'),
                    print(LS),nl,
                    print('--------------------------------------------------------------------------'),nl,
                    halt
                |existsOptArg('-h','--help',Opts) ->
                    print_help(Desc_CL),
                    halt
                | Args==[] ->
                    print_help(Desc_CL),
                    halt
                |otherwise ->
                    % Setting enviroment
                    getOptArg('-k','--knowledge_base',Opts,'KB.pl',KBFile),
                    getOptArg('','--rht_acts',Opts,'rht_acts.pl',RhtActsFile),
                    getOptArg('','--sp_acts',Opts,'sp_acts.pl',SpActsSpeechFile),
                    getOptArg('','--vs_acts',Opts,'vs_acts.pl',SpActsVisionFile),
                    getOptArg('','--templates',Opts,'templates.pl',TemplatesFile),
                    getOptArg('','--motor_actions',Opts,'motor_actions.pl',MotorActionsFile),
                    getOptArg('','--user_functions',Opts,'user_functions.pl',UserFunctionsFile),
                    getOptArg('-m','--mode',Opts,run,Mode),
                    getOptArg('-l','--log',Opts,'dm.log',LogFile),
                    getOptArg('-v','--verbose',Opts,'inflog',Verbose),
                    getOptArg('project_dir','',Args,'',ProjDir),
                    getOptArg('main_dialogue','',Args,'',MainDiag),
                    getOptArg('dialogue_list_file','',Args,'',ModelFiles),

                    assert(mode_exe(Mode)),
                    print(Mode),
                    (mode_exe(run) ->
                        start,
                        print('Executing using OAA2'),
                        nl
                        | otherwise ->
                        print('Executing test'),
                        nl
                    ),
                    create_log_file(ProjDir,Verbose,LogFile),

                    write_log(inflog,'Running models from     : ~w',[ModelFiles]),
                    write_log(inflog,'Project directory       : ~w',[ProjDir]),
                    write_log(inflog,'Knowledge base          : ~w',[KBFile]),
                    write_log(inflog,'User Functions          : ~w',[UserFunctionsFile]),
                    write_log(inflog,'Rhetorical acts         : ~w',[RhtActsFile]),
                    write_log(inflog,'Speech Acts             : ~w',[SpActsSpeechFile]),
                    write_log(inflog,'Vision Acts             : ~w',[SpActsVisionFile]),
                    write_log(inflog,'Motor Actions           : ~w',[MotorActionsFile]),
                    write_log(inflog,'Templates               : ~w',[TemplatesFile]),
                    write_log(inflog,'Running dialogue        : ~w',[MainDiag]),
                    write_log(inflog,'Writing log to          : ~w',[LogFile]),
                   
                    

                    assert(user:file_search_path(projdir,ProjDir)),
                    % Loading files from project directory 
                    consult(projdir(RhtActsFile)),
                    consult(projdir(SpActsSpeechFile)),
                    consult(projdir(SpActsVisionFile)),
                    consult(projdir(TemplatesFile)),
                    consult(projdir(MotorActionsFile)),
                    consult(projdir(UserFunctionsFile)),
                    consult(projdir(KBFile)),
   
                    % Loading dialogue files
                    load_list_of_files(ProjDir, ModelFiles, Dialogues),
                    assert(dialogues(Dialogues)),

                    %load language models
                    %load_file(ProjDir,LanguageModelFile,Lang_Model),
                    %assert(language_models(Lang_Model)),

                    %History is the sequence of pairs diag_ID:situation
                    %visited during the dialogue model interpretation
                    write_log(inflog,'Starting dialogue: ~w',[MainDiag]),            
                    execute_diagmod(MainDiag,[],History),

                    % Dialogue finishes
                    write_log(inflog,'Ending dialogue: ~w',[MainDiag]),
                    print('Historia del Recorrido: '),nl,
                    print_history(History),nl,
                    write_log(inflog,'History: ~w.',[History]),
                    reset)
            | otherwise ->
                print_help(Desc_CL)),
                halt
        | otherwise -> 
            print_help(Desc_CL),
            halt
        ).
	   
print_history([]) :- 
    print('fin del recorrido'),nl.
print_history([H|T]) :-
	print(H),nl,
    print_history(T).


reset :-
           retract(language_models(_)),
           retract(paths(_)).

% Executes a dialogue model
execute_diagmod(Dialogue_ID, In_history, New_history) :-
    write_log(inflog,'Executing dialogue: ~w',[Dialogue_ID]),
    dialogues(Dialogues),
    get_dialogue(Dialogue_ID, Dialogues, Situations, Speech_Acts),
    get_structure(is, Situations, Current_Situation),
    write_log(infolog,'Situtation: is',[]),
    write_log(structlog,'Executing situtation: ~w',[Current_Situation]),
    % Executes initial situation (is)
    execute_situation(Dialogue_ID, In_history, Out_history,
                          Speech_Acts, empty, Current_Situation, Out_Pair),
    % Moves to the next situation
    next_situation(Dialogue_ID, Out_history, New_history,
                       Speech_Acts, is, Situations, Out_Pair),!.

% Error, the dialogue does not exists
execute_diagmod(Dialogue_ID,_,_) :- 
    write_log(errlog,'There is not dialogue: ~w',[Dialogue_ID]),
    warning_msg(dialogue,Dialogue_Id).

% It arrives to final situation
next_situation(_, History, History ,_, fs, _, Out_Pair) :-
    write_log(inflog,'Situtaion: fs',[]),
    Out_Pair =.. [=>,_,[]].

% It arrives to a good situation
next_situation(Dialogue_ID, In_history, Out_history, Speech_Acts, ID_Previous, Situations, In_Pair) :-
    In_Pair =.. [=>, In_Trans, ID_Current_Pred],
    write_log(inflog,'Situation: ~w',[ID_Current_Pred]),
    get_structure(ID_Current_Pred, Situations, Current_Sit),

    write_log(structlog,'Executing situtation: ~w',[Current_Sit]),
    execute_situation(Dialogue_ID, In_history, New_history,
                              Speech_Acts, ID_Previous => In_Trans, Current_Sit, Out_Pair),

    eval_situation(Dialogue_ID,New_history,Out_Pair,EOut_Pair),

    next_situation(Dialogue_ID, New_history, Out_history,
                           Speech_Acts, ID_Current_Pred, Situations, EOut_Pair).

%Get Dialogue: Dialogue_ID, Dialogues, Current_Dialogue
get_dialogue(ID,Dialogues,Sits,SpActs):-
	write_log(deblog,'Looking for ~w',[ID]),
    (atom(ID) -> 
        ID_temp  = ID
    | 
	    ID =.. [ID_pred|Pars_pred],
        new_vars(Pars_pred,Pars_temp),
        ID_temp =.. [ID_pred|Pars_temp]
    ),
	write_log(deblog,'Testing with template ~w',[ID_temp]),
    get_dialogue_(ID_temp,ID,Dialogues,Sits,SpActs).


get_dialogue_(ID,_,[],_,no_existe) :- warning_msg(dialogue,ID).
get_dialogue_(ID,Real_ID,[diag_mod(ID,Situaciones,Speech_Acts)|_],Situaciones_Cloned,Speech_Acts_Cloned):-
	write_log(structlog,'Dialogue ~w is a candidate',[ID]),
    vars_ls_structure(Situaciones,[],Vars),
    vars_ls_structure(Speech_Acts,Vars,NVars_),
	write_log(structlog,'Collected vars: ~w',[NVars_]),
    new_vars(NVars_,NNVars_),
    ( atom(ID) ->
	    write_log(deblog,'AAA ~w',[ID]),
        ID_ = Real_ID,
        NVars = NVars_,
        NNVars = NNVars_
    | otherwise ->  
	    write_log(deblog,'BBB ~w',[ID]),
        Real_ID =.. [ID_|PARS_],
        ID      =.. [ID__|PARS__],
        plug_vars(PARS__,PARS_,NVars_,NNVars__,NVars,NNVars)
        
    ),

	write_log(structlog,'New vars : ~w',[NNVars]),
    clone_ls_structure(Situaciones,Situaciones_Cloned,[NVars,NNVars]),
    clone_ls_structure(Speech_Acts,Speech_Acts_Cloned,[NVars,NNVars]),

	write_log(structlog,'Clone candidates situations ~w ',[Situaciones_Cloned]),
	write_log(structlog,'Clone candidates sp acts ~w ',[Speech_Acts_Cloned]).

get_dialogue_(ID,Real_ID,[_|Rest],Situaciones_C,Speech_Acts_C) :-
    get_dialogue_(ID,Real_ID,Rest,Situaciones_C,Speech_Acts_C).


new_vars([],[]).
new_vars([A|L],[NA|NL]):-
    new_vars(L,NL),!.

plug_vars([],[],V,NV,V,NV).
plug_vars([O|Os],[P|Ps],V,NV,RV,RNV):-
    plug_var(O,P,V,NV),
    plug_vars(Os,Ps,V,NV,RV,RNV),!.

plug_var(O,P,[],[]).
plug_var(O,P,[O_|Os],[N_|Ns]):-
    O== O_,
    N_=P,
    plug_var(O,P,Os,Ns),!.
plug_var(O,P,[O_|Os],[N_|Ns]):-
    plug_var(O,P,Os,Ns),!.




%Get Situation, Speech Act or Basic Rhetorical Act: Structure_ID, Structures, Current_Structure
get_structure(ID,Sits,Res):-
	write_log(deblog,'Looking for ~w',[ID]),
    (atom(ID)  ->
        ID_temp  = ID
    | 
        ID =.. [ID_pred|Pars_pred],
        new_vars(Pars_pred,Pars_temp),
        ID_temp =.. [ID_pred|Pars_temp]
	),
	write_log(deblog,'Looking the template ~w',[ID_temp]),
    get_structure_(ID_temp,ID,Sits,Res).



% Error, no structure
get_structure_(ID_temp,ID_pred,[],no_existe) :- warning_msg(structure,ID).
% It finds a structure
get_structure_(ID_temp,ID_pred,[Structure|_],Clone) :-
	write_log(structlog,'Comparing with ~w',[Structure]),
    get_feature_value(id,Structure,ID_temp),
	write_log(structlog,'It is a candidate',[]),
    vars_structure(Structure,[],Vars),
	write_log(structlog,'Collected vars: ~w',[Vars]),
    get_new_variables(Vars,NVars),
	write_log(structlog,'New vars : ~w',[NVars]),
    clone_structure(Structure,Clone,[Vars,NVars]),
	write_log(structlog,'Clone candidate ~w',[Clone]),
    get_feature_value(id,Clone,ID_pred),!.


% Collects vars from a structure


vars_structure([],Vars,Vars).
vars_structure([ID==>Val|Rest],KVars,RVars):-
    vars_val(Val,KVars,Vars),
    vars_structure(Rest,Vars,RVars),!.
vars_ls_structure([],KVars,KVars).
vars_ls_structure([St|Rest],KVars,RVars):-
    vars_structure(St,KVars,Vars),
    vars_ls_structure(Rest,Vars,RVars),!.
    

% There is nothing to copy anymore
clone_structure([],[],_).
clone_structure([ID==>Val|Rest],[ID==>NVal|NRest],Vars):-
    clone_val(Val,NVal,Vars),
    clone_structure(Rest,NRest,Vars),!.
clone_ls_structure([],[],Vars).
clone_ls_structure([St|Rest],[NSt|NRest],Vars):-
    clone_structure(St,NSt,Vars),
    clone_ls_structure(Rest,NRest,Vars),!.



% atoms
vars_val(Atom,KVars,KVars):-
    atom(Atom).

clone_val(Atom,Atom,Vars):-
    atom(Atom).


% vars
vars_val(Var,KVars,Res):-
    var(Var),
    (check_var(Var,KVars,true) ->
        Res = KVars
    |
        Res = [Var|KVars]
    ).

check_var(Var,[],false).
check_var(Var,[NVar|Rest],true):-
    Var==NVar,!.
check_var(Var,[NVar|Rest],Res):-
    check_var(Var,Rest,Res),!.

clone_val(Var,NVar,[Vars,NVars]):-
    var(Var),
    get_nth(Var,0,Vars,ID),
    nth(ID,NVars,NVar),!.

get_nth(_,_,[],-1).
get_nth(Var,Acc,[NVar|Rest],ID):-
    Var==NVar,
    ID is Acc + 1.
get_nth(Var,Acc,[NVar|Rest],Res):-
    ID is Acc + 1,
    get_nth(Var,ID,Rest,Res).

% Arcs
vars_val(SpAct:Rht=>Dest,KVars,NVars):-
    vars_val(SpAct,KVars,NVars1),
    vars_val(Rht,NVars1,NVars2),
    vars_val(Dest,NVars2,NVars),!.

clone_val(SpAct:Rht=>Dest,NSpAct:NRht=>NDest,Vars):-
    clone_val(SpAct,NSpAct,Vars),
    clone_val(Rht,NRht,Vars),
    clone_val(Dest,NDest,Vars).


% lists
vars_val([],KVars,KVars).
vars_val([V|R],KVars,NVars):-
    vars_val(V,KVars,Vars),
    vars_val(R,Vars,NVars).


clone_val([],[],Vars).
clone_val([V|R],[NV|NR],Vars):-
    clone_val(V,NV,Vars),
    clone_val(R,NR,Vars).


% Preds
vars_val(PredArgs,KVars,NVars):-
    PredArgs =.. [Pred|Args],
    vars_val(Args,KVars,NVars).

clone_val(PredArgs,NPredArgs,Vars) :-
    PredArgs =.. [Pred|Args],
    clone_val(Args,NArgs,Vars),
    NPredArgs =..[Pred|NArgs].

% It looks for the next one
get_structure(ID,[_|Rest],Structure) :- 
    get_structure(ID,Rest,Structure).

get_feature_value(Feature,[],Def,Def).
get_feature_value(Feature,[Feature ==> Value|Rest],Def,Value):-!.
get_feature_value(Feature,[_|Rest], Def,Value) :-
    get_feature_value(Feature,Rest,Def,Value),!.

%Get Feature Value: Feature, Situation or Speech Act, Feature's value
get_feature_value(Feature,[],no_existe) :- warning_msg(feature,Feature).
get_feature_value(Feature,[Feature ==> Value|Rest],Value):-!.
get_feature_value(Feature,[_|Rest], Value) :-
    get_feature_value(Feature,Rest,Value),!.

%Return the history of transitions in current dialogue model
%Parameters: Dialogue_Model_ID, Global_History, DM_History
get_current_DM_history(ID, [], []).
get_current_DM_history(ID, [(ID:STEP)|T], [(ID:STEP)|Rest]) :-
	get_current_DM_history(ID, T, Rest).
get_current_DM_history(ID, [(ID2:_)|T],Local_History) :-
	get_current_DM_history(ID, T, Local_History).

% Executing situations


%Execute neutral situation
execute_situation(Dialogue_ID, In_history, Out_history, Speech_Acts,In_Pair,Situation,Out_Pair) :-
    get_feature_value(type,Situation,neutral),
    get_feature_value(id,Situation,ID),
    write_log(deblog,'In situation ~w of type ~w.',[ID,neutral]),
    write_log(debstruct,'Situation to execute ~w.',[Situation]),

    get_feature_value(out_pairs,Situation,[Def_Out_Pair|_]),
    eval_intentions(Dilaogue_ID,In_history,[Def_Out_Pair],[Def_Out_Pair_Eval],Expected),

    %execute rhetorical act
    write_log(structlog,'In_pair : ~w',[In_Pair]),
    write_log(structlog,'Arc : ~w',[Def_Out_Pair]),
    action_situation(Dialogue_ID, In_history,
                  Speech_Acts, In_Pair, Def_Out_Pair_Eval, Out_Pair),

    write_log(structlog,'Out_pair: ~w',[Out_Pair]),

    %register history
    append(In_history,[Dialogue_ID:(In_Pair,ID,Out_Pair)],Out_history),!.

%Execute final situation
execute_situation(Dialogue_ID, In_history, Out_history, Speech_Acts,In_Pair,Situation,Out_Pair) :-
    %Check final situation
    get_feature_value(type,Situation,final),
    get_feature_value(id,Situation,ID),
    write_log(deblog,'In situation ~w of type ~w.',[ID,final]),

    get_feature_value(out_pairs,Situation,[Out_Pair|_]),

    %register history
    append(In_history,[Dialogue_ID:(In_Pair,fs,Out_Pair)],Out_history),!.



%Execute listening situation
execute_situation(Dialogue_ID, In_history, Out_history, Speech_Acts, In_Pair, Situation, Out_Pair) :-
    %Check listening situation
    get_feature_value(type,Situation,listening),
    get_feature_value(id,Situation,ID),
    write_log(deblog,'In situation ~w of type ~w.',[ID,listening]),
    write_log(debstruct,'Situation to execute ~w.',[Situation]),
    get_feature_value(out_pairs, Situation, Out_Pairs),
    eval_intentions(Dialogue_ID,In_history,Out_Pairs,Out_Pairs_Eval,Exp_Intentions),

    %call input text or speech agent
    (mode_exe(test) ->
        print('Expected Intentions: '),print(Exp_Intentions),nl,
        read(In_Speech_Act)
    | otherwise -> 
        get_speech_act(Exp_Intentions,In_Speech_Act)
    ),

    write_log(inflog,'Input speech act: ~w',[In_Speech_Act]),
    %check input token is a x of a x:ra in Out_Pairs
    %and return Out_Pair

    %select the out pair as originally defined in the language model
    recognize_speech_act(In_Speech_Act, Out_Pairs_Eval, Selected_Out_Pair),

    write_log(inflog,'Output arc: ~w',[Selected_Out_Pair]),
    action_situation(Dialogue_ID, In_history, Speech_Acts, In_Pair, Selected_Out_Pair, Out_Pair),

    %register history            
    append(In_history, [Dialogue_ID:(In_Pair,ID,Out_Pair)], Out_history),!.

% Excecute seeing situation
execute_situation(Dialogue_ID, In_history, Out_history, Speech_Acts, In_Pair, Situation, Out_Pair) :-
    get_feature_value(type,Situation,seeing),
    get_feature_value(id,Situation,ID),
    write_log(deblog,'In situation ~w of type ~w.',[ID,seeing]),
    write_log(debstruct,'Situation to execute ~w.',[Situation]),
    get_feature_value(out_pairs, Situation, Out_Pairs),
    eval_intentions(Dialogue_ID, In_history, Out_Pairs,Out_Pairs_Eval,Exp_Intentions),

    (mode_exe(test) ->
        print('Expected Intentions: '),print(Exp_Intentions),nl,
        read(In_Speech_Act)
    | otherwise -> 
        get_vision_act(Exp_Intentions,In_Speech_Act)
    ),

     write_log(inflog,'Input visual act: ~w',[In_Speech_Act]),
    %check input token is a x of a x:ra in Out_Pairs
    %and return Out_Pair

    %select the out pair as originally defined in the language model
    recognize_speech_act(In_Speech_Act, Out_Pairs_Eval, Selected_Out_Pair),

    write_log(inflog,'Output arc: ~w',[Selected_Out_Pair]),
    action_situation(Dialogue_ID, In_history, Speech_Acts, In_Pair, Selected_Out_Pair, Out_Pair),

    %register history
    append(In_history, [Dialogue_ID:(In_Pair,ID,Out_Pair)], Out_history),!.


recognize_speech_act(unknown, [], unexpected_speech_act).
recognize_speech_act(Sp_Act, [], unexpected_speech_act).
recognize_speech_act(Sp_Act, [Sp_Act:Rht_Act => Next|Rest],Sp_Act:Rht_Act => Next).
recognize_speech_act(Sp_Act, [_|Rest], Out_Pair) :-
    recognize_speech_act(Sp_Act, Rest, Out_Pair).


%error situation
execute_situation(Dialogue_ID, In_history, Out_history,
                  Speech_Acts,ID_Previous => In_Trans, Situation, Out_Pair) :-
	%Check situation type and id
    get_feature_value(type,Situation,error),
    get_feature_value(id,Situation,unexpected_speech_act),
	write_log(deblog,'In situation ~w of type ~w.',['unexpected_speech_act',error]),

    %execute rhetorical act
    perform_rht_act(Speech_Acts, error(misunderstanding)),

    Out_Pair =.. [=>, In_Trans, ID_Previous],

	%register history
	append(In_history, [Dialogue_ID:(ID_Previous => In_Trans,unexpected_speech_act,Out_Pair)], Out_history),!.

%Execute recursive situation
execute_situation(Dialogue_ID, In_history, Out_history, 
		    Speech_Acts, Prev_Sit => In_Speech_Act:In_Rht_Act, Situation,Out_Pair) :-

    %Check situation type
    get_feature_value(type,Situation,recursive(Dialogue)),
    get_feature_value(id,Situation,ID),
	write_log(deblog,'In situation ~w of type ~w.',[ID,recursive]),

    execute_diagmod(Dialogue, In_history, New_history),

    get_feature_value(out_pairs,Situation,[Out_Pair|_]),

    Out_Pair =.. [=>, _:Rht_Act, _],
    %execute out rhetorical act
    perform_rht_act(Speech_Acts, Rht_Act),

    %register history            
    append(New_history, [Dialogue_ID:((Prev_Sit => In_Speech_Act:In_Rht_Act),ID,Out_Pair)], Out_history), !.


%Eval intention arcs
eval_intentions(Dialogue_ID,History,[],[],[]).
eval_intentions(Dialogue_ID,History,[Intention|RIntentions],[Out_Intention|ROut_Intentions],[Expected|RExpected]):-
    eval_intention(Dialogue_ID,History,Intention,Out_Intention,Expected),
    eval_intentions(Dialogue_ID,History,RIntentions,ROut_Intentions,RExpected).


eval_intention(Dialogue_ID,History,apply(Func,Local_Arg):Rht=>Sit,Intention:Rht=>Sit,Intention):-
    eval_function(Func, [Dialogue_ID, History, Local_Arg],Intention),
	write_log(deblog,'Going to situation ~w',[Out_Sit]).
eval_intention(Dialogue_ID,History,Intention:Rht=>Sit,Intention:Rht=>Sit,Intention).



%Eval situations arcs
eval_situation(Dialogue_ID,History,In_Pair,Out_Pair):-
    In_Pair =.. [=>, Var, apply(Func,Local_Arg)],
    eval_function(Func, [Dialogue_ID, History, Local_Arg],Out_Sit),
    Out_Pair =.. [=>, Var, Out_Sit],
	write_log(deblog,'Going to situation ~w',[Out_Sit]).


eval_situation(Dialogue_ID,History,In_Pair,Out_Pair):-
    In_Pair =.. [=>, Var, Sit],
    Out_Pair =.. [=>, Var, Sit],
	write_log(deblog,'Going to situation ~w',[Sit]).

%error input speech act
action_situation(Dialogue_ID,History,_, In_Pair, unexpected_speech_act, Out_Pair) :-
    In_Pair =.. [=>, _, In_Trans],
    Out_Pair =.. [=>, In_Trans, unexpected_speech_act],
    write_log(deblog,'Action ~w',[unexpected_speech_act]).

%Rhetorial act defined as a function
action_situation(Dialogue_ID,History, Speech_Acts,In_Pair, Out_Pair, Out_Pair) :-
    Out_Pair =.. [=>, Sp_Act:apply(Func,Local_Arg), Next],            
    write_log(debug,'Performing fun ~w with local arguments~w',[Func,Local_Arg]),
    eval_function(Func,[Dialogue_ID,History, Local_Arg],Out_Rht_Act),

    write_log(deblog,'Out_Rht_Act: ~w',[Out_Rht_Act]),

    %execute rhetorical act
    perform_rht_act(Speech_Acts, Out_Rht_Act),
	write_log(deblog,'Action as predicate ~w',[Rht_Act]).


%Rhetorical act defined as a grounded predicated
action_situation(Dialague_ID,History, Speech_Acts,In_Pair, Out_Pair, Out_Pair) :-
    Out_Pair =.. [=>, Sp_Act:Rht_Act, Next], 
    ground_term(Rht_Act,true),

    write_log(debug,'Performing rht act: ~w',[Rht_Act]),
    %execute rhetorical act
    perform_rht_act(Speech_Acts, Rht_Act),
    write_log(deblog,'Action ~w',[Rht_Act]).

perform_rht_act(Speech_Acts, Rht_Act) :-
    get_structure(Rht_Act, Speech_Acts, Speech_Act),
    get_feature_value(rht_acts,Speech_Act,[],Rht_Acts),
    %realize basic rethorical acts
    mode_exe(Val),
    write_log(inflog,"Executing rhetorical acts ~w",[Rht_Acts]),
    ( mode_exe(test) ->
        print('Executing rhetorical acts '),
        print(Rht_Acts),nl,
        rht_act_realization(Rht_Acts)
    | otherwise -> 
        rht_act_realization(Rht_Acts)
    ).



get_function((Func,Local_Arg), Func, Local_Arg).
eval_function(Name,Pars, Func_Value) :-
    Func =..[Name|Pars],
    Func,

    %regresa el valor de la funcion
    (retract(function_value(Func_Value));
        print('error in function definition: No Value defined'),nl).

%Errores:
warning_msg(dialogue,ID) :-
    write_log(warlog,'No such Dialogue: ~w',[ID]),!.

warning_msg(structure,ID) :-           
    write_log(warlog,'No such Situation, Speech Act or basic Rht. Act. in Dialogue: ~w',[ID]),!.

warning_msg(feature,Feature) :-           
    write_log(warlog,'No such feature in Structure: ~w',[Feature]),!.

error_msg(speech_act,ID) :-
    write_log(errlog,'Unexpected Speech Act: ~w',[ID]),!.

warning_msg(language_model,ID) :-
    write_log(warlog,'No such language model: ~w',[ID]),!.

