% Loads logging functions
% Copyright (c) 2009 Ivan Meza (http://turing.iimas.unam.mx/~ivanvladimir)

:- use_module(library(system)).

create_log_file(ProjDir,ModeLabel,LogFile):-
	modenum(ModeLabel,Mode,_),
        (absolute_file_name(LogFile, AbsOutputFile,[relative_to(ProjDir)]),
             open(AbsOutputFile,write,Stream)) ->
             format('Log file:~n    ~w~n~n',
                     [AbsOutputFile]),
	     user:assert(log_stream(Stream,user)),
	     user:assert(log_mode(Mode))
         | otherwise ->
             format('Specified log file (~w)~n', [LogFile]),
             format('    cannot be written.  Using stderr', []),
             user:assert(log_stream(user_error,sys)),
	     user:assert(log_mode(Mode))
         .


modenum(errlog  ,0,'Error').
modenum(inflog  ,1,'Info-Level 1').
modenum(inflog2 ,2,'Info-Level 2').
modenum(deblog  ,3,'Debug').
modenum(structlog,4,'Structs').
modenum(X       ,5,'All').

write_log(ModeLabel,MSG,Args):-
	log_mode(LogMode),
	modenum(ModeLabel,Mode,SLog),
	((Mode =< LogMode) ->
	   log_stream(Stream,_),
       datime(datime(Year,Month,Day,Hour,Min,Sec)),
	   format(Stream,'[~w/~w/~w,~w:~w:~w]~w: ',[Year,Month,Day,Hour,Min,Sec,SLog]),
	   format(Stream,MSG,Args),
	   format(Stream,'~n',[]),
	   flush_output(Stream)
	| otherwise ->
	   true).

test_logging:-
	create_log_file(errlog,file),
	write_log(errlog,'Logged',[]),
	write_log(log,'No logged',[]).

